import org.mortbay.jetty.handler.*;
import org.mortbay.jetty.*;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

/**
 * Обработчик запросов.
 */
public class AddressBookHandler extends AbstractHandler {
    private AddressBook _addressBook = null;

    public AddressBookHandler(AddressBook aAddressBook) {
        _addressBook = aAddressBook;
    }

    public void handle(String aTarget, HttpServletRequest aRequest, HttpServletResponse aResponse, int aDispatchMode)
            throws IOException, ServletException {
        ((Request) aRequest).setHandled(true);
        aRequest.setCharacterEncoding("utf-8");
        aResponse.setCharacterEncoding("utf-8");
        aResponse.addHeader("Content-type", "text/html; charset=utf-8");

        PrintWriter writer = aResponse.getWriter();
        String styles = outputStyles();
        writer.write("<html><head>" +
                "<title>Адресная книга</title>" +
                styles +
                "</head><body><h1>Адресная книга</h1>");

        if ("/".equals(aTarget)) {
            writer.write("<a href=\"/add\">Добавить запись</a><br/>");
            writer.write("<a href=\"/view\">Просмотреть записи</a><br/>");
        } else {
            writer.write("<a href=\"/\">На главную</a><br/>");

            if ("/add".equals(aTarget)) {
                writer.write("<h2>Добавление записи</h2>");
                handleAdd(aRequest, writer);
            } else if ("/view".equals(aTarget)) {
                writer.write("<h2>Просмотр записей</h2>");
                handleView(writer);
            } else if ("/edit".equals(aTarget)) {
                writer.write("<h2>Редактирование записи</h2>");
                handleEdit(aRequest, writer);
            } else if ("/remove".equals(aTarget)) {
                writer.write("<h2>Удаление записи</h2>");
                handleRemove(aRequest, writer);
            }
        }

        writer.write("</body></html>");
    }

    private void handleRemove(HttpServletRequest aRequest, PrintWriter aWriter) throws IOException {
        if (aRequest.getParameter("number") != null) {
            _addressBook.removeContactByNumber(aRequest.getParameter("number"));
            outputMessage(aWriter, "Удален контакт с номером " + aRequest.getParameter("number"));
        }

        handleView(aWriter);
    }

    private void handleEdit(HttpServletRequest aRequest, PrintWriter aWriter) throws IOException {
        if (aRequest.getParameter("number") == null) {
            _addressBook.removeContactByNumber(aRequest.getParameter("number"));
            outputMessage(aWriter, "Не определено, что редактировать");
            handleView(aWriter);
        } else if (aRequest.getParameter("edited") != null) {
            _addressBook.editContact(aRequest.getParameter("edited"),
                    aRequest.getParameter("name"), aRequest.getParameter("number"), aRequest.getParameter("comment"));
            outputMessage(aWriter, "Контакт отредактирован");
            handleView(aWriter);
        } else {
            Contact contact = _addressBook.getContactByNumber(aRequest.getParameter("number"));
            outputForm(aWriter, aRequest, contact.getName(), contact.getNumber(), contact.getComment());
        }
    }

    private void handleAdd(HttpServletRequest aRequest, PrintWriter aWriter) throws IOException {
        if (aRequest.getParameter("name") != null) {
            _addressBook.addContact(aRequest.getParameter("name"), aRequest.getParameter("number"), aRequest.getParameter("comment"));
            outputMessage(aWriter, "Контакт добавлен");
        }
        outputForm(aWriter, aRequest, "", "", "");
    }

    private void handleView(PrintWriter aWriter) {
        aWriter.write("<table border==\"1\">");
        aWriter.write("<tr><td width=\"100\">Имя</td><td width=\"100\">Номер</td><td width=\"100\">Комментарий</td><td> - </td></tr>");

        SortedMap<String,Contact> sortedMap = _addressBook.getContactsByName();
        for (Contact contact : sortedMap.values()) {
            aWriter.write(
                    "<tr>" +
                        "<td class=\"name\">" + contact.getName() + "</td>" +
                        "<td class=\"number\">" + contact.getNumber() + "</td>" +
                        "<td class=\"comment\">" + contact.getComment() + "</td>" +
                        "<td class=\"name\">" +
                            "<a href=\"/remove?number=" + contact.getNumber() + "\">Удалить</a> " +
                            "<a href=\"/edit?number=" + contact.getNumber() + "\">Редактировать</a>" +
                        "</td>" +
                    "</tr>");
        }

        aWriter.write("</table>");
    }

    private void outputForm(PrintWriter aWriter, HttpServletRequest aRequest,
                            String aName, String aNumber, String aComment) {
        aWriter.write(
                "<form action=\"/edit\" method=\"post\"><input type=\"hidden\" name=\"edited\" value=\"" + aRequest.getParameter("number") + "\"/>" +
                        "<table>" +
                        "<tr><td>Имя: </td><td><input type=\"text\" name=\"name\" value=\"" + aName + "\"/></td></tr>" +
                        "<tr><td>Телефон: </td><td><input type=\"text\" name=\"number\" value=\"" + aNumber + "\"/></td></tr>" +
                        "<tr><td>Примечания: </td><td><input type=\"text\" name=\"comment\" value=\"" + aComment + "\"/></td></tr>" +
                        "<tr><td colspan=\"2\" align=\"center\"><input type=\"submit\" name=\"Отправить\"/></td></tr>" +
                        "</table>" +
                        "</form>");
    }

    private void outputMessage(PrintWriter aWriter, String aMessage) {
        aWriter.write("<span style=\"color: green;\">" + aMessage + "</span>");
    }

    private String outputStyles() {
        return "<style type=\"text/css\">" +
                "</style>";
    }
}
